use std::{io::{BufReader, BufRead, self}, fs::File, str::FromStr, path::Path, convert::Infallible, num::ParseIntError};

#[derive(Debug)]
pub struct AdventError(pub String);

impl From<ParseIntError> for AdventError {
    fn from(e: ParseIntError) -> Self {
        AdventError(e.to_string())
    }
}

impl From<io::Error> for AdventError {
    fn from(e: io::Error) -> Self {
        AdventError(e.to_string())
    }
}

impl From<Infallible> for AdventError {
    fn from(_: Infallible) -> Self {
        unreachable!()
    }
}

pub fn read_into_vec<T:FromStr>(file_path : &Path) -> Result<Vec<T>, AdventError> where AdventError: From<<T as FromStr>::Err> {
    let file = File::open(file_path).expect("no such file");
    let buf = BufReader::new(file);
    buf.lines().map(|line| T::from_str(line?.as_str()).map_err(Into::into)).collect::<Result<Vec<T>, AdventError>>()
}